"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const disposable_1 = require("@reactive-js/disposable");
const option_1 = require("@reactive-js/option");
class IteratorEnumerator extends disposable_1.AbstractDisposable {
    constructor(iterator) {
        super();
        this.iterator = iterator;
        this.current = option_1.none;
        this.hasCurrent = false;
    }
    move(_) {
        this.hasCurrent = false;
        this.current = option_1.none;
        const next = this.iterator.next();
        if (!next.done) {
            this.hasCurrent = true;
            this.current = next.value;
        }
        else {
            this.dispose();
        }
        return this.hasCurrent;
    }
}
class IteratorEnumerable {
    constructor(f) {
        this.f = f;
    }
    enumerate() {
        const iterator = this.f();
        const enumerator = new IteratorEnumerator(iterator);
        enumerator.add(error => {
            if (option_1.isSome(error) && option_1.isSome(iterator.throw)) {
                const { cause } = error;
                iterator.throw(cause);
            }
            else if (option_1.isSome(iterator.return)) {
                iterator.return();
            }
        });
        return enumerator;
    }
}
exports.fromIterator = (f) => new IteratorEnumerable(f);
exports.fromIterable = (iterable) => exports.fromIterator(() => iterable[Symbol.iterator]());
//# sourceMappingURL=fromIterator.js.map