import { AbstractDisposable } from "@reactive-js/disposable";
import { isSome, none } from "@reactive-js/option";
class IteratorEnumerator extends AbstractDisposable {
    constructor(iterator) {
        super();
        this.iterator = iterator;
        this.current = none;
        this.hasCurrent = false;
    }
    move(_) {
        this.hasCurrent = false;
        this.current = none;
        const next = this.iterator.next();
        if (!next.done) {
            this.hasCurrent = true;
            this.current = next.value;
        }
        else {
            this.dispose();
        }
        return this.hasCurrent;
    }
}
class IteratorEnumerable {
    constructor(f) {
        this.f = f;
    }
    enumerate() {
        const iterator = this.f();
        const enumerator = new IteratorEnumerator(iterator);
        enumerator.add(error => {
            if (isSome(error) && isSome(iterator.throw)) {
                const { cause } = error;
                iterator.throw(cause);
            }
            else if (isSome(iterator.return)) {
                iterator.return();
            }
        });
        return enumerator;
    }
}
export const fromIterator = (f) => new IteratorEnumerable(f);
export const fromIterable = (iterable) => fromIterator(() => iterable[Symbol.iterator]());
//# sourceMappingURL=fromIterator.js.map